package com.sxc.etaxincome.common.util;

import com.sxc.framework.common.exception.HttpClientException;
import com.sxc.framework.common.httpclient.HttpHelper;
import com.sxc.framework.common.httpclient.ResponseContent;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.NoHttpResponseException;
import org.apache.http.conn.ConnectTimeoutException;

import java.nio.charset.StandardCharsets;
import java.util.Map;

@Slf4j
public class RetryHttpHelper {
    private RetryHttpHelper() {

    }
    public static final int COLLECTION_REQUEST_TIMEOUT_MILLIS = 5000;
    public static final int CONNECTION_TIMEOUT_MILLS = 5000;
    public static final int SOCKET_TIMEOUT_MILLIS = 90000;

    /**
     * retry times
     */
    public static final int RETRY_TIMES = 3;

    public static final int WAIT_SECONDS = 2;

    public static ResponseContent retryPostUrl(String url, Map<String, Object> paramsMap) {
        return retryPostUrl(url, paramsMap, CONNECTION_TIMEOUT_MILLS, SOCKET_TIMEOUT_MILLIS);
    }

    public static ResponseContent retryPostUrl(String url, Map<String, Object> paramsMap, Integer connectTimeoutMilliSeconds, Integer socketTimeoutMilliSeconds) {
        RetryUtil<ResponseContent> retryUtil = RetryUtil.<ResponseContent>builder().times(RETRY_TIMES).waitSeconds(WAIT_SECONDS).build();
        try {
            return retryUtil.retry(() -> postUrl(url, paramsMap, connectTimeoutMilliSeconds, socketTimeoutMilliSeconds));
        } catch (Exception e) {
            throw new HttpClientException(e.getCause().getMessage());
        }
    }

    private static ResponseContent postUrl(String url, Map<String, Object> paramsMap, Integer connectTimeoutMilliSeconds, Integer socketTimeoutMilliSeconds) {
        return postUrl(() -> HttpHelper.postUrl(url, paramsMap, connectTimeoutMilliSeconds, socketTimeoutMilliSeconds));
    }

    public static ResponseContent retryPostJsonEntity(String url, String jsonBody) {
        return retryPostJsonEntity(url, jsonBody, COLLECTION_REQUEST_TIMEOUT_MILLIS, CONNECTION_TIMEOUT_MILLS, SOCKET_TIMEOUT_MILLIS);
    }

    public static ResponseContent retryPostJsonEntity(String url, String jsonBody, int connectionRequestTimeout, int connectTimeout, int socketTimeout) {
        RetryUtil<ResponseContent> retryUtil = RetryUtil.<ResponseContent>builder().times(RETRY_TIMES).waitSeconds(WAIT_SECONDS).build();
        try {
            return retryUtil.retry(() -> postJsonEntity(url, jsonBody, connectionRequestTimeout, connectTimeout, socketTimeout));
        } catch (Exception e) {
            throw new HttpClientException(e.getCause().getMessage());
        }
    }

    private static ResponseContent postJsonEntity(String url, String jsonBody, int connectionRequestTimeout, int connectTimeout, int socketTimeout) {
        return postUrl(() -> HttpHelper.postJsonEntity(url, jsonBody, connectionRequestTimeout, connectTimeout, socketTimeout));
    }

    private static ResponseContent postUrl(ExecuteRequestCallback executeRequestCallback) {
        try {
            return executeRequestCallback.execute();
        } catch (HttpClientException e) {
            /*
            HttpClientException是HttpHelper工具封装的异常
             */
            log.error("execute makeout invoice http request error:", e);
            /*
            NoHttpResponseException 重试
            ConnectTimeoutException 连接超时重试
            SocketTimeoutException 响应超时不重试，避免造成业务数据不一致
             */
            Throwable causeException = e.getCause();
            /*
            NoHttpResponseException说明
            httpclient之前与服务端建立的链接已经失效（例如：tomcat 默认的keep-alive timeout ：20s），
            再次从连接池拿该失效链接进行请求时，就会报错。
             */
            String causeExceptionMessage = causeException.getMessage();
            if (causeException instanceof NoHttpResponseException
                    || causeException instanceof ConnectTimeoutException) {
                log.error("retry exception");
                throw new HttpClientException(causeExceptionMessage);
            } else {
                log.error("cause exception message:{}", causeExceptionMessage);
                ResponseContent responseContent = new ResponseContent();
                responseContent.setContentBytes(causeExceptionMessage.getBytes(StandardCharsets.UTF_8));
                return responseContent;
            }
        }
    }

    interface ExecuteRequestCallback {
        ResponseContent execute();
    }

}
