package com.sxc.etaxincome.domain.command.query;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.nacos.api.config.annotation.NacosValue;
import com.google.common.base.Stopwatch;
import com.google.common.collect.Lists;
import com.sxc.etaxincome.common.util.RetryHttpHelper;
import com.sxc.etaxincome.domain.business.IApiConfigManager;
import com.sxc.etaxincome.domain.constants.EtaxReportConstants;
import com.sxc.etaxincome.domain.exception.EtaxIncomeException;
import com.sxc.etaxincome.domain.model.common.CommonRequestWrapper;
import com.sxc.etaxincome.domain.model.common.PageList;
import com.sxc.etaxincome.domain.model.common.CommonResponse;
import com.sxc.etaxincome.domain.model.cookie.TokenNecessary;
import com.sxc.etaxincome.domain.model.report.ApiConfig;
import com.sxc.etaxincome.domain.model.report.CancelReportResponse;
import com.sxc.etaxincome.domain.model.report.IReportResponse;
import com.sxc.etaxincome.domain.model.report.QueryCancelListRequestWrapper;
import com.sxc.etaxincome.domain.template.ResponseHandlerTemplate;
import com.sxc.etaxincome.enums.ApiCode;
import com.sxc.framework.common.httpclient.ResponseContent;
import com.sxc.framework.common.json.JSONUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.concurrent.TimeUnit;

import static com.sxc.etaxincome.domain.constants.EtaxReportConstants.*;

@Slf4j
@Component(value = "queryCancelListQueryPageListCommand")
public class QueryCancelListQueryPageListCommand implements IQueryPageListCommand {

    @NacosValue(value = "${url.etax.report}", autoRefreshed = true)
    private String etaxReportUrl;

    @Autowired
    private IApiConfigManager apiConfigManager;

    @Override
    public CommonResponse<PageList<IReportResponse>> execute(CommonRequestWrapper commonRequestWrapper) {
        Stopwatch stopwatch = Stopwatch.createStarted();
        ApiConfig apiConfig = apiConfigManager.getApiConfig(ApiCode.QUERY_CANCEL_PAGE_LIST.getCode());
        JSONObject requestDataJsonObject = new JSONObject();
        String requestDataStr = commonRequestWrapper.getData();
        String requestSerialNo = commonRequestWrapper.getRequestSerialNo();
        log.info("queryCancelPageList requestSerialNo:{} data:{}", requestSerialNo, requestDataStr);
        QueryCancelListRequestWrapper queryCancelListRequestWrapper = JSON.parseObject(requestDataStr, QueryCancelListRequestWrapper.class);
        TokenNecessary tokenNecessary = queryCancelListRequestWrapper.decodeToken();
        String token = tokenNecessary.getToken();
        requestDataJsonObject.put(API_PARAM_TOKEN, token);
        requestDataJsonObject.put(API_PARAM_AREA_CODE, queryCancelListRequestWrapper.getArea());
        String test = queryCancelListRequestWrapper.getTest();
        if (StringUtils.isNotBlank(test)) {
            requestDataJsonObject.put(API_PARAM_TEST, test);
        }
        if (StringUtils.isNotBlank(queryCancelListRequestWrapper.getReportDateStart())) {
            requestDataJsonObject.put("sbrqq", queryCancelListRequestWrapper.getReportDateStart());
        }
        if (StringUtils.isNotBlank(queryCancelListRequestWrapper.getReportDateEnd())) {
            requestDataJsonObject.put("sbrqz", queryCancelListRequestWrapper.getReportDateEnd());
        }
        if (queryCancelListRequestWrapper.getPageSize() != null) {
            requestDataJsonObject.put("pageSize", queryCancelListRequestWrapper.getPageSize().toString());
        }
        if (queryCancelListRequestWrapper.getPageNumber() != null) {
            requestDataJsonObject.put("pageNumber", queryCancelListRequestWrapper.getPageNumber().toString());
        }
        try {
            String etaxReportUrlFull = etaxReportUrl + apiConfig.getApiUrl();
            log.info("url:{}", etaxReportUrlFull);
            String requestJson = JSONUtil.toString(requestDataJsonObject);
            log.info("request:{}", requestJson);
            ResponseContent responseContent = RetryHttpHelper.retryPostJsonEntity(etaxReportUrlFull, requestJson);
            String response = responseContent.getContent();
            log.info("response:{}", response);
            stopwatch.stop();
            log.info("query cancel page list elapsed time:{}ms", stopwatch.elapsed(TimeUnit.MILLISECONDS));
            return new ResponseHandlerTemplate<PageList<IReportResponse>>() {
                @Override
                public CommonResponse<PageList<IReportResponse>> handleSuccess(JSONObject jsonObject) {
                    JSONObject resultJsonObject = jsonObject.getJSONObject(EtaxReportConstants.ETAX_RESPONSE_DATA_KEY);
                    JSONArray jsonArray = resultJsonObject.getJSONArray("List");
                    Integer total = resultJsonObject.getInteger("Total");
                    Integer pageSize = resultJsonObject.getInteger("PageSize");
                    Integer pageNumber = resultJsonObject.getInteger("PageNumber");
                    List<IReportResponse> invoiceResponseList = Lists.newArrayListWithExpectedSize(10);
                    for (int i = 0; i < jsonArray.size(); i++) {
                        JSONObject jsonObjectInvoice = jsonArray.getJSONObject(i);
                        CancelReportResponse cancelReportResponse = new CancelReportResponse();
                        cancelReportResponse.putAll(jsonObjectInvoice);
                        invoiceResponseList.add(cancelReportResponse);
                    }
                    PageList<IReportResponse> pageList = PageList.<IReportResponse>builder()
                            .total(total)
                            .pageNumber(pageNumber)
                            .pageSize(pageSize)
                            .list(invoiceResponseList).build();
                    return CommonResponse.success(pageList);
                }
            }.handle(response);
        } catch (Exception e) {
            log.error("query cancel page list error:", e);
            throw new EtaxIncomeException(e);
        }
    }
}
